@@ -87,19 +87,22 @@ class QrackSystem:
             CFUNCTYPE(c_ulonglong, c_double, c_double),
         ]
 
-        # These next two methods need to have c_double pointers, if PyQrack is built with fp64.
+        # These next few methods need to have c_double pointers, if PyQrack is built with fp64.
         self.qrack_lib.InKet.restype = None
         self.qrack_lib.OutKet.restype = None
         self.qrack_lib.OutProbs.restype = None
+        self.qrack_lib.OutReducedDensityMatrix.restype = None
 
         if self.fppow < 6:
             self.qrack_lib.InKet.argtypes = [c_ulonglong, POINTER(c_float)]
             self.qrack_lib.OutKet.argtypes = [c_ulonglong, POINTER(c_float)]
             self.qrack_lib.OutProbs.argtypes = [c_ulonglong, POINTER(c_float)]
+            self.qrack_lib.OutReducedDensityMatrix.argtypes = [c_ulonglong, c_ulonglong, POINTER(c_ulonglong), POINTER(c_float)]
         else:
             self.qrack_lib.InKet.argtypes = [c_ulonglong, POINTER(c_double)]
             self.qrack_lib.OutKet.argtypes = [c_ulonglong, POINTER(c_double)]
             self.qrack_lib.OutProbs.argtypes = [c_ulonglong, POINTER(c_double)]
+            self.qrack_lib.OutReducedDensityMatrix.argtypes = [c_ulonglong, c_ulonglong, POINTER(c_ulonglong), POINTER(c_double)]
 
         self.qrack_lib.init.restype = c_ulonglong
         self.qrack_lib.init.argtypes = []
@@ -143,8 +146,24 @@ class QrackSystem:
         self.qrack_lib.set_concurrency.restype = None
         self.qrack_lib.set_concurrency.argtypes = [c_ulonglong, c_ulonglong]
 
+        self.qrack_lib.set_device.restype = None
+        self.qrack_lib.set_device.argtypes = [c_ulonglong, c_longlong]
+
+        self.qrack_lib.set_device_list.restype = None
+        self.qrack_lib.set_device_list.argtypes = [
+            c_ulonglong,
+            c_ulonglong,
+            POINTER(c_longlong),
+        ]
+
         # pseudo-quantum
 
+        self.qrack_lib.HighestProbAll.restype = None
+        self.qrack_lib.HighestProbAll.argtypes = [c_ulonglong, POINTER(c_ulonglong)]
+
+        self.qrack_lib.HighestProbAllN.restype = None
+        self.qrack_lib.HighestProbAllN.argtypes = [c_ulonglong, c_ulonglong, POINTER(c_ulonglong)]
+
         self.qrack_lib.ProbAll.restype = None
         if self.fppow == 5:
             self.qrack_lib.ProbAll.argtypes = [
@@ -780,6 +799,9 @@ class QrackSystem:
         self.qrack_lib.MAll.restype = c_ulonglong
         self.qrack_lib.MAll.argtypes = [c_ulonglong]
 
+        self.qrack_lib.MAllLong.restype = None
+        self.qrack_lib.MAllLong.argtypes = [c_ulonglong, POINTER(c_ulonglong)]
+
         self.qrack_lib.Measure.restype = c_ulonglong
         self.qrack_lib.Measure.argtypes = [
             c_ulonglong,
